@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Plus42 -- an enhanced HP-42S calculator simulator
- * Copyright (C) 2004-2022  Thomas Okken
+ * Copyright (C) 2004-2025  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -27,7 +27,6 @@
 #include "core_math1.h"
 #include "core_sto_rcl.h"
 #include "core_variables.h"
-#include "shell.h"
 
 
 /********************************************************/
@@ -588,21 +587,21 @@ int docmd_cla(arg_struct *arg) {
 int docmd_deg(arg_struct *arg) {
     flags.f.grad = 0;
     flags.f.rad = 0;
-    shell_annunciators(-1, -1, -1, -1, 0, 0);
+    set_annunciators(-1, -1, -1, -1, 0, 0);
     return ERR_NONE;
 }
 
 int docmd_rad(arg_struct *arg) {
     flags.f.grad = 0;
     flags.f.rad = 1;
-    shell_annunciators(-1, -1, -1, -1, 0, 1);
+    set_annunciators(-1, -1, -1, -1, 0, 1);
     return ERR_NONE;
 }
 
 int docmd_grad(arg_struct *arg) {
     flags.f.grad = 1;
     flags.f.rad = 0;
-    shell_annunciators(-1, -1, -1, -1, 1, 1);
+    set_annunciators(-1, -1, -1, -1, 1, 1);
     return ERR_NONE;
 }
 
@@ -623,9 +622,10 @@ int docmd_size(arg_struct *arg) {
         return ERR_INVALID_TYPE;
     vloc idx = lookup_var("REGS", 4);
     if (arg->val.num == 0) {
-        if (!idx.not_found())
-            purge_var("REGS", 4, true);
-        return ERR_NONE;
+        if (!idx.not_found() && !purge_var("REGS", 4, true))
+            return ERR_RESTRICTED_OPERATION;
+        else
+            return ERR_NONE;
     }
     if (idx.not_found()) {
         vartype *regs = new_realmatrix(arg->val.num, 1);
@@ -727,6 +727,8 @@ static int clv_helper(arg_struct *arg, bool global) {
         return ERR_INVALID_TYPE;
     if (!purge_var(arg->val.text, arg->length, false, global, !global))
         return ERR_RESTRICTED_OPERATION;
+    if (!global)
+        maybe_pop_indexed_matrix(arg->val.text, arg->length);
     remove_shadow(arg->val.text, arg->length);
     return ERR_NONE;
 }
@@ -793,6 +795,8 @@ int docmd_del(arg_struct *arg) {
         return ERR_INVALID_TYPE;
     if (!current_prgm.is_editable())
         return ERR_RESTRICTED_OPERATION;
+    if (current_prgm.is_locked())
+        return ERR_PROGRAM_LOCKED;
     clear_prgm_lines(arg->val.num);
     return ERR_NONE;
 }
@@ -1189,9 +1193,9 @@ static int mappable_rnd_c(phloat xre, phloat xim, phloat *yre, phloat *yim) {
 
 int round_easy(phloat x, phloat *y) {
     if (flags.f.fix_or_all && flags.f.eng_or_all) {
-	// This is the internal rounding used by AMORT; unlike the RND
-	// function, which rounds to 12 digits in ALL mode, I think it makes
-	// more sense not to round at all in the case of AMORT steps.
+        // This is the internal rounding used by AMORT; unlike the RND
+        // function, which rounds to 12 digits in ALL mode, I think it makes
+        // more sense not to round at all in the case of AMORT steps.
         *y = x;
         return ERR_NONE;
     }
