@@ -177,8 +177,6 @@ int docmd_comb(arg_struct *arg) {
         x = y - x;
     #ifdef BCD_MATH
         s = x == 0 ? 1 : pow(10, 1 + floor(log10(x)));
-    #elif ANDROID
-        s = x == 0 ? 1 : pow(2, 1 + floor(log(x) / log(phloat(2))));
     #else
         s = x == 0 ? 1 : pow(2, 1 + floor(log2(x)));
     #endif
@@ -1944,6 +1942,7 @@ int docmd_off(arg_struct *arg) {
     if (program_running() && no_keystrokes_yet)
         return ERR_SUSPICIOUS_OFF;
     set_running(false);
+    quitting = true;
     shell_powerdown();
     return ERR_NONE;
 }
