@@ -252,8 +252,10 @@ int docmd_insr(arg_struct *arg) {
     return ERR_NONE;
 }
 
-static void invrt_completion(vartype *res) {
-    unary_result(res);
+static int invrt_completion(int error, vartype *res) {
+    if (error == ERR_NONE)
+        unary_result(res);
+    return error;
 }
 
 int docmd_invrt(arg_struct *arg) {
