@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Plus42 -- an enhanced HP-42S calculator simulator
- * Copyright (C) 2004-2022  Thomas Okken
+ * Copyright (C) 2004-2025  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -20,10 +20,10 @@
 #include "core_commands6.h"
 #include "core_commands8.h"
 #include "core_helpers.h"
+#include "core_main.h"
 #include "core_math2.h"
 #include "core_sto_rcl.h"
 #include "core_variables.h"
-#include "shell.h"
 
 /********************************************************/
 /* Implementations of HP-42S built-in functions, part 6 */
@@ -392,27 +392,23 @@ static int mappable_log_c(phloat xre, phloat xim, phloat *yre, phloat *yim) {
             *yim = 0;
         } else {
             *yre = log10(-xre);
-            *yim = PI / log(10.0);
+            *yim = PI / log(phloat(10.0));
         }
         return ERR_NONE;
     } else if (xre == 0) {
         if (xim > 0) {
             *yre = log10(xim);
-            *yim = PI / log(100.0);
+            *yim = PI / log(phloat(100.0));
         } else {
             *yre = log10(-xim);
-            *yim = -PI / log(100.0);
+            *yim = -PI / log(phloat(100.0));
         }
         return ERR_NONE;
     } else {
-        phloat h = hypot(xre, xim);
-        if (p_isinf(h)) {
-            const phloat s = 10000;
-            h = hypot(xre / s, xim / s);
-            *yre = log10(h) + 4;
-        } else
-            *yre = log10(h);
-        *yim = atan2(xim, xre) / log(10.0);
+        math_ln(xre, xim, yre, yim);
+        phloat s = log(phloat(10.0));
+        *yre /= s;
+        *yim /= s;
         return ERR_NONE;
     }
 }
@@ -426,7 +422,7 @@ int docmd_log(arg_struct *arg) {
             if (flags.f.real_result_only)
                 return ERR_INVALID_DATA;
             else {
-                vartype *r = new_complex(log10(-x->x), PI / log(10.0));
+                vartype *r = new_complex(log10(-x->x), PI / log(phloat(10.0)));
                 if (r == NULL)
                     return ERR_INSUFFICIENT_MEMORY;
                 else {
@@ -465,7 +461,7 @@ static int mappable_10_pow_x_r(phloat x, phloat *y) {
 static int mappable_10_pow_x_c(phloat xre, phloat xim, phloat *yre, phloat *yim){
     int inf;
     phloat h;
-    xim *= log(10.0);
+    xim *= log(phloat(10.0));
     if ((inf = p_isinf(xim)) != 0)
         xim = inf < 0 ? NEG_HUGE_PHLOAT : POS_HUGE_PHLOAT;
     h = pow(10, xre);
@@ -511,40 +507,6 @@ static int mappable_ln_r(phloat x, phloat *y) {
     }
 }
 
-static int mappable_ln_c(phloat xre, phloat xim, phloat *yre, phloat *yim) {
-    if (xim == 0) {
-        if (xre == 0)
-            return ERR_INVALID_DATA;
-        if (xre > 0) {
-            *yre = log(xre);
-            *yim = 0;
-        } else {
-            *yre = log(-xre);
-            *yim = PI;
-        }
-        return ERR_NONE;
-    } else if (xre == 0) {
-        if (xim > 0) {
-            *yre = log(xim);
-            *yim = PI / 2;
-        } else {
-            *yre = log(-xim);
-            *yim = -PI / 2;
-        }
-        return ERR_NONE;
-    } else {
-        phloat h = hypot(xre, xim);
-        if (p_isinf(h)) {
-            const phloat s = 10000;
-            h = hypot(xre / s, xim / s);
-            *yre = log(h) + log(s);
-        } else
-            *yre = log(h);
-        *yim = atan2(xim, xre);
-        return ERR_NONE;
-    }
-}
-
 int docmd_ln(arg_struct *arg) {
     if (stack[sp]->type == TYPE_REAL) {
         vartype_real *x = (vartype_real *) stack[sp];
@@ -573,7 +535,7 @@ int docmd_ln(arg_struct *arg) {
         }
     } else {
         vartype *v;
-        int err = map_unary(stack[sp], &v, mappable_ln_r, mappable_ln_c);
+        int err = map_unary(stack[sp], &v, mappable_ln_r, math_ln);
         if (err == ERR_NONE)
             unary_result(v);
         return err;
@@ -725,51 +687,6 @@ static int mappable_inv_r(phloat x, phloat *y) {
     return ERR_NONE;
 }
 
-static int mappable_inv_c(phloat xre, phloat xim, phloat *yre, phloat *yim) {
-    phloat r, t, rre, rim;
-    int inf;
-    if (xre == 0 && xim == 0)
-        return ERR_DIVIDE_BY_0;
-    if (fabs(xim) <= fabs(xre)) {
-        r = xim / xre;
-        t = 1 / (xre + xim * r);
-        if (r == 0) {
-            rre = t;
-            rim = -xim * (1 / xre) * t;
-        } else {
-            rre = t;
-            rim = -r * t;
-        }
-    } else {
-        r = xre / xim;
-        t = 1 / (xre * r + xim);
-        if (r == 0) {
-            rre = xre * (1 / xim) * t;
-            rim = -t;
-        } else {
-            rre = r * t;
-            rim = -t;
-        }
-    }
-    inf = p_isinf(rre);
-    if (inf != 0) {
-        if (flags.f.range_error_ignore)
-            rre = inf == 1 ? POS_HUGE_PHLOAT : NEG_HUGE_PHLOAT;
-        else
-            return ERR_OUT_OF_RANGE;
-    }
-    inf = p_isinf(rim);
-    if (inf != 0) {
-        if (flags.f.range_error_ignore)
-            rim = inf == 1 ? POS_HUGE_PHLOAT : NEG_HUGE_PHLOAT;
-        else
-            return ERR_OUT_OF_RANGE;
-    }
-    *yre = rre;
-    *yim = rim;
-    return ERR_NONE;
-}
-
 int docmd_inv(arg_struct *arg) {
     vartype *x = stack[sp];
     vartype *v;
@@ -781,7 +698,7 @@ int docmd_inv(arg_struct *arg) {
         err = unit_div(x, one, &v);
         free_vartype(one);
     } else
-        err = map_unary(stack[sp], &v, mappable_inv_r, mappable_inv_c);
+        err = map_unary(stack[sp], &v, mappable_inv_r, math_inv);
     if (err == ERR_NONE)
         unary_result(v);
     return err;
@@ -816,19 +733,14 @@ int docmd_y_pow_x(arg_struct *arg) {
         return ERR_INVALID_TYPE;
     } else if (stack[sp]->type == TYPE_REAL) {
         phloat x = ((vartype_real *) stack[sp])->x;
-        if (x == floor(x)) {
+        if (x == to_int4(x)) {
             /* Integer exponent */
             if (stack[sp - 1]->type == TYPE_REAL) {
                 /* Real number to integer power */
                 phloat y = ((vartype_real *) stack[sp - 1])->x;
-                if (x == 0 && y == 0)
+                if (y == 0 && x <= 0)
                     return ERR_INVALID_DATA;
                 phloat r = pow(y, x);
-                if (p_isnan(r))
-                    /* Should not happen; pow() is supposed to be able
-                     * to raise negative numbers to integer exponents
-                     */
-                    return ERR_INVALID_DATA;
                 if ((inf = p_isinf(r)) != 0) {
                     if (!flags.f.range_error_ignore)
                         return ERR_OUT_OF_RANGE;
@@ -838,8 +750,68 @@ int docmd_y_pow_x(arg_struct *arg) {
                 goto done;
             } else {
                 /* Complex number to integer power */
-                phloat rre, rim, yre, yim;
-                int4 ex;
+                phloat yre = ((vartype_complex *) stack[sp - 1])->re;
+                phloat yim = ((vartype_complex *) stack[sp - 1])->im;
+                int4 ex = to_int4(x);
+
+                if (yim == 0) {
+                    /* Pure real base */
+                    if (yre == 0 && ex <= 0)
+                        return ERR_INVALID_DATA;
+                    phloat r = pow(yre, ex);
+                    if ((inf = p_isinf(r)) != 0) {
+                        if (!flags.f.range_error_ignore)
+                            return ERR_OUT_OF_RANGE;
+                        r = inf < 0 ? NEG_HUGE_PHLOAT : POS_HUGE_PHLOAT;
+                    }
+                    res = new_complex(r, 0);
+                    goto done;
+                } else if (yre == 0) {
+                    /* Pure imaginary base */
+                    phloat r = pow(yim, ex);
+                    if ((inf = p_isinf(r)) != 0) {
+                        if (!flags.f.range_error_ignore)
+                            return ERR_OUT_OF_RANGE;
+                        r = inf < 0 ? NEG_HUGE_PHLOAT : POS_HUGE_PHLOAT;
+                    }
+                    switch (ex & 3) {
+                        case 0: res = new_complex( r,  0); break;
+                        case 1: res = new_complex( 0,  r); break;
+                        case 2: res = new_complex(-r,  0); break;
+                        case 3: res = new_complex( 0, -r); break;
+                    }
+                    goto done;
+                }
+
+                bool invert = ex < 0;
+                if (invert)
+                    ex = -ex;
+                phloat rre, rim;
+
+                if (yre == yim || yre == -yim) {
+                    /* (a±ai)^n => pure real, pure imaginary, or r±ri */
+#ifdef BCD_MATH
+                    phloat r = pow(yre * yre * 2, ex / 2);
+                    if (ex % 2 != 0)
+                        r *= yre;
+#else
+                    phloat r = scalbn(pow(yre, ex), ex / 2);
+#endif
+                    switch (ex % 8) {
+                        case 0: rre =  r; rim =  0; break;
+                        case 1: rre =  r; rim =  r; break;
+                        case 2: rre =  0; rim =  r; break;
+                        case 3: rre = -r; rim =  r; break;
+                        case 4: rre = -r; rim =  0; break;
+                        case 5: rre = -r; rim = -r; break;
+                        case 6: rre =  0; rim = -r; break;
+                        case 7: rre =  r; rim = -r; break;
+                    }
+                    if (yre != yim)
+                        rim = -rim;
+                    goto complex_pow_int_done;
+                }
+
                 if (x < -256.0 || x > 256.0)
                     /* At some point, the cumulative error in repeated squaring
                      * becomes larger than the error in the polar power
@@ -849,20 +821,11 @@ int docmd_y_pow_x(arg_struct *arg) {
                      * return exact results for short bases, i.e. bases where
                      * first and last nonzero digits are close together. The
                      * smallest non-trivial base is 1+i, which can be raised to
-                     * the power 225 in the decimal version, and to the power
-                     * 107 in the binary version, with exact results.
+                     * the power 225 in the decimal version with exact results.
                      */
                     goto complex_pow_real_1;
                 rre = 1;
                 rim = 0;
-                yre = ((vartype_complex *) stack[sp - 1])->re;
-                yim = ((vartype_complex *) stack[sp - 1])->im;
-                ex = to_int4(x);
-                if (ex <= 0 && yre == 0 && yim == 0)
-                    return ERR_INVALID_DATA;
-                bool invert = ex < 0;
-                if (invert)
-                    ex = -ex;
                 while (1) {
                     phloat tmp;
                     if ((ex & 1) != 0) {
@@ -886,10 +849,11 @@ int docmd_y_pow_x(arg_struct *arg) {
                     if (p_isinf(yre) || p_isnan(yre) || p_isinf(yim) || p_isnan(yim))
                         goto complex_pow_real_1;
                 }
+                complex_pow_int_done:
                 if (invert) {
-                    phloat h = hypot(rre, rim);
-                    rre = rre / h / h;
-                    rim = (-rim) / h / h;
+                    int err = math_inv(rre, rim, &rre, &rim);
+                    if (err != ERR_NONE)
+                        return err;
                 }
                 if ((inf = p_isinf(rre)) != 0) {
                     if (!flags.f.range_error_ignore)
@@ -982,7 +946,7 @@ int docmd_y_pow_x(arg_struct *arg) {
                 res = new_complex(0, 0);
             goto done;
         }
-        err = mappable_ln_c(yre, yim, &lre, &lim);
+        err = math_ln(yre, yim, &lre, &lim);
         if (err != ERR_NONE)
             return err;
         tmp = lre * xre - lim * xim;
@@ -1120,7 +1084,10 @@ int docmd_stoflag(arg_struct *arg) {
     if (new_rad == old_rad)
         new_rad = -1;
     if (new_g != -1 || new_rad != -1)
-        shell_annunciators(-1, -1, -1, -1, new_g, new_rad);
+        set_annunciators(-1, -1, -1, -1, new_g, new_rad);
+
+    if (b <= 30 && e >= 30 && ((lfs >> 30) & 1) == 1)
+        mode_disable_stack_lift = true;
 
     return ERR_NONE;
 }
