@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Plus42 -- an enhanced HP-42S calculator simulator
- * Copyright (C) 2004-2022  Thomas Okken
+ * Copyright (C) 2004-2025  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -56,6 +56,7 @@ int docmd_eval(arg_struct *arg) {
         current_prgm.set(eq_dir->id, eq->data->eqn_index);
         pc = 0;
         store_stack_reference((vartype *) eq);
+        save_csld();
         return ERR_NONE;
     } else {
         clear_all_rtns();
@@ -63,6 +64,7 @@ int docmd_eval(arg_struct *arg) {
         current_prgm.set(eq_dir->id, eq->data->eqn_index);
         pc = 0;
         store_stack_reference((vartype *) eq);
+        save_csld();
         return ERR_RUN;
     }
 }
@@ -81,6 +83,7 @@ int docmd_evaln(arg_struct *arg) {
         current_prgm.set(eq_dir->id, eq->data->eqn_index);
         pc = 0;
         store_stack_reference((vartype *) eq);
+        save_csld();
         return ERR_NONE;
     } else {
         clear_all_rtns();
@@ -88,6 +91,7 @@ int docmd_evaln(arg_struct *arg) {
         current_prgm.set(eq_dir->id, eq->data->eqn_index);
         pc = 0;
         store_stack_reference((vartype *) eq);
+        save_csld();
         return ERR_RUN;
     }
 }
@@ -156,13 +160,15 @@ int docmd_xeql(arg_struct *arg) {
             int4 dummy2;
             bool dummy3;
             pop_rtn_addr(&dummy1, &dummy2, &dummy3);
-        }
+        } else
+            save_csld();
         return err;
     } else {
         int err = docmd_gtol(arg);
         if (err != ERR_NONE)
             return err;
         clear_all_rtns();
+        save_csld();
         return ERR_RUN;
     }
 }
@@ -259,27 +265,15 @@ static int item_helper(arg_struct *arg, bool get) {
         }
     }
 
-    phloat d = ((vartype_real *) stack[spo])->x;
-    if (d <= -2147483648.0 || d >= 2147483648.0)
-        return ERR_DIMENSION_ERROR;
-    int4 n = to_int4(d);
-    if (n < 0)
-        n = -n;
-    n--;
-    if (n < 0)
+    int4 n;
+    if (!dim_to_int4(stack[spo], &n))
         return ERR_DIMENSION_ERROR;
 
     if (two_d) {
         if (v->type == TYPE_LIST)
             return ERR_DIMENSION_ERROR;
-        d = ((vartype_real *) stack[spo - 1])->x;
-        if (d <= -2147483648.0 || d >= 2147483648.0)
-            return ERR_DIMENSION_ERROR;
-        int4 m = to_int4(d);
-        if (m < 0)
-            m = -m;
-        m--;
-        if (m < 0)
+        int4 m;
+        if (!dim_to_int4(stack[spo - 1], &m))
             return ERR_DIMENSION_ERROR;
         int4 cols = v->type == TYPE_REALMATRIX
                 ? ((vartype_realmatrix *) v)->columns
@@ -524,15 +518,60 @@ int docmd_to_par(arg_struct *arg) {
     return store_params();
 }
 
-int docmd_fdepth(arg_struct *arg) {
-    int depth;
-    int err = get_frame_depth(&depth);
+int docmd_fstart(arg_struct *arg) {
+    if (!program_running())
+        return ERR_RESTRICTED_OPERATION;
+    if (!need_fstart())
+        return ERR_NONE;
+    int err = push_func_state(1);
     if (err != ERR_NONE)
         return err;
-    vartype *v = new_real(depth);
+    if (flags.f.big_stack)
+        return ERR_NONE;
+    else
+        return push_stack_state(true);
+}
+
+int docmd_fstack(arg_struct *arg) {
+    phloat plevel = ((vartype_real *) stack[sp])->x;
+    if (plevel < 0)
+        plevel = -plevel;
+    int4 level;
+    if (plevel >= 2147483648.0)
+        level = 2147483647;
+    else
+        level = to_int4(plevel);
+    vartype *res;
+    int err = get_saved_stack_level(level, &res);
+    if (err == ERR_NONE)
+        unary_result(res);
+    return err;
+}
+
+int docmd_embed(arg_struct *arg) {
+    equation_data *eqd = eq_dir->prgms[arg->val.num].eq_data;
+    vartype *v = new_equation(eqd);
     if (v == NULL)
         return ERR_INSUFFICIENT_MEMORY;
-    return recall_result(v);
+    if (arg->type == ARGTYPE_NUM)
+        return recall_result(v);
+    int err = push_rtn_addr(current_prgm, pc);
+    if (err != ERR_NONE)
+        return err;
+    current_prgm.set(eq_dir->id, eqd->eqn_index);
+    pc = 0;
+    store_stack_reference(v);
+    free_vartype(v);
+    save_csld();
+    return ERR_NONE;
+}
+
+int docmd_neweqn(arg_struct *arg) {
+    return eqn_new();
+}
+
+int docmd_editeqn(arg_struct *arg) {
+    return eqn_edit();
 }
 
 static int get_sum(int n) {
@@ -645,40 +684,40 @@ const unitdef units[] = {
     { "\21", "m", 1, 1, -6 },
     { "\24", "m", 1, 1, -10 },
     { "fermi", "m", 1, 1, -15 },
-    { "m^2", NULL, 1, 1, 0 },
-    { "cm^2", "m^2", 1, 1, -4 },
-    { "b", "m^2", 1, 1, -28 },
-    { "yd^2", "m^2", 83612736, 1, -8 },
-    { "ft^2", "m^2", 9290304, 1, -8 },
-    { "in^2", "m^2", 64516, 1, -8 },
-    { "km^2", "m^2", 1, 1, 6 },
-    { "ha", "m^2", 1, 1, 4 },
-    { "a", "m^2", 1, 1, 2 },
-    { "mi^2", "m^2", 2589988110336, 1, -6 },
-    { "miUS^2", "m^2", 40144896, 15499969, 6 },
-    { "acre", "m^2", 627264, 15499969, 5 },
-    { "m^3", NULL, 1, 1, 0 },
-    { "st", "m^3", 1, 1, 0 },
-    { "cm^3", "m^3", 1, 1, -6 },
-    { "yd^3", "m^3", 764554857984, 1, -12 },
-    { "ft^3", "m^3", 28316846592, 1, -12 },
-    { "in^3", "m^3", 16387064, 1, -12 },
-    { "l", "m^3", 1, 1, -3 },
-    { "galUK", "m^3", 4546092, 1, -9 },
-    { "galC", "m^3", 454609, 1, -8 },
-    { "gal", "m^3", 3785411784LL, 1, -12 },
-    { "qt", "m^3", 946352946, 1, -12 },
-    { "pt", "m^3", 473176473, 1, -12 },
-    { "ml", "m^3", 1, 1, -6 },
-    { "cu", "m^3", 2365882365LL, 1, -13 },
-    { "ozfl", "m^3", 295735295625, 1, -16 },
-    { "ozUK", "m^3", 28413075, 1, -12 },
-    { "tbsp", "m^3", 1478676478125, 1, -17 },
-    { "tsp", "m^3", 492892159375, 1, -17 },
-    { "bbl", "m^3", 158987294928, 1, -12 }, // 42 gal
-    { "bu", "m^3", 3523907, 1, -8 },
-    { "pk", "m^3", 88097675, 1, -10 },
-    { "fbm", "m^3", 2359737216LL, 1, -12 }, // 144 in^3
+    { "m\0362", NULL, 1, 1, 0 },
+    { "cm\0362", "m\0362", 1, 1, -4 },
+    { "b", "m\0362", 1, 1, -28 },
+    { "yd\0362", "m\0362", 83612736, 1, -8 },
+    { "ft\0362", "m\0362", 9290304, 1, -8 },
+    { "in\0362", "m\0362", 64516, 1, -8 },
+    { "km\0362", "m\0362", 1, 1, 6 },
+    { "ha", "m\0362", 1, 1, 4 },
+    { "a", "m\0362", 1, 1, 2 },
+    { "mi\0362", "m\0362", 2589988110336, 1, -6 },
+    { "miUS\0362", "m\0362", 40144896, 15499969, 6 },
+    { "acre", "m\0362", 627264, 15499969, 5 },
+    { "m\0363", NULL, 1, 1, 0 },
+    { "st", "m\0363", 1, 1, 0 },
+    { "cm\0363", "m\0363", 1, 1, -6 },
+    { "yd\0363", "m\0363", 764554857984, 1, -12 },
+    { "ft\0363", "m\0363", 28316846592, 1, -12 },
+    { "in\0363", "m\0363", 16387064, 1, -12 },
+    { "l", "m\0363", 1, 1, -3 },
+    { "galUK", "m\0363", 4546092, 1, -9 },
+    { "galC", "m\0363", 454609, 1, -8 },
+    { "gal", "m\0363", 3785411784LL, 1, -12 },
+    { "qt", "m\0363", 946352946, 1, -12 },
+    { "pt", "m\0363", 473176473, 1, -12 },
+    { "ml", "m\0363", 1, 1, -6 },
+    { "cu", "m\0363", 2365882365LL, 1, -13 },
+    { "ozfl", "m\0363", 295735295625, 1, -16 },
+    { "ozUK", "m\0363", 28413075, 1, -12 },
+    { "tbsp", "m\0363", 1478676478125, 1, -17 },
+    { "tsp", "m\0363", 492892159375, 1, -17 },
+    { "bbl", "m\0363", 158987294928, 1, -12 }, // 42 gal
+    { "bu", "m\0363", 3523907, 1, -8 },
+    { "pk", "m\0363", 88097675, 1, -10 },
+    { "fbm", "m\0363", 2359737216LL, 1, -12 }, // 144 in^3
     { "yr", "s", 315569259747, 1, -4 }, // tropical year
     { "d", "s", 864, 1, 2 },
     { "h", "s", 36, 1, 2 },
@@ -692,14 +731,14 @@ const unitdef units[] = {
     { "mph", "m/s", 44704, 1, -5 },
     { "knot", "m/s", 463, 900, 0 },
     { "c", "m/s", 299792458, 1, 0 },
-    { "ga", "m/s^2", 980665, 1, -5 },
+    { "ga", "m/s\0362", 980665, 1, -5 },
     { "kg", NULL, 1, 1, 0 },
     { "g", "kg", 1, 1, -3 },
     { "lb", "kg", 45359237, 1, -8 },
     { "oz", "kg", 28349523125, 1, -12 },
     { "slug", "kg", 145939029372, 1, -10 },
     { "lbt", "kg", 3732417216LL, 1, -10 },
-    { "ton", "kg", 90718474, 1, 5 }, // 2000 lb
+    { "ton", "kg", 90718474, 1, -5 }, // 2000 lb
     { "tonUK", "kg", 10160469088, 1, -7 },
     { "t", "kg", 1, 1, 3 },
     { "ozt", "kg", 311034768, 1, -10 },
@@ -707,70 +746,70 @@ const unitdef units[] = {
     { "grain", "kg", 6479891, 1, -11 },
     { "u", "kg", 16605402, 1, -34 },
     { "mol", NULL, 1, 1, 0 },
-    { "N", "kg*m/s^2", 1, 1, 0 },
-    { "dyn", "kg*m/s^2", 1, 1, -5 },
-    { "gf", "kg*m/s^2", 980665, 1, -8 },
-    { "kip", "kg*m/s^2", 444822161526, 1, -8 },
-    { "lbf", "kg*m/s^2", 444822161526, 1, -11 },
-    { "pdl", "kg*m/s^2", 138254954376, 1, -12 },
-    { "J", "kg*m^2/s^2", 1, 1, 0 },
-    { "erg", "kg*m^2/s^2", 1, 1, -7 },
-    { "kcal", "kg*m^2/s^2", 41868, 1, -1 },
-    { "cal", "kg*m^2/s^2", 41868, 1, -4 },
-    { "Btu", "kg*m^2/s^2", 105505585262, 1, -8 },
-    { "ft*lbf", "kg*m^2/s^2", 135581794833, 1, -11 },
-    { "therm", "kg*m^2/s^2", 105506, 1, 3 },
-    { "MeV", "kg*m^2/s^2", 160217733, 1, -21 },
-    { "eV", "kg*m^2/s^2", 160217733, 1, -27 },
-    { "W", "kg*m^2/s^3", 1, 1, 0 },
-    { "hp", "kg*m^2/s^3", 745699871582, 1, -9 },
-    { "Pa", "kg/(m*s^2)", 1, 1, 0 },
-    { "atm", "kg/(m*s^2)", 101325, 1, 0 },
-    { "bar", "kg/(m*s^2)", 100000, 1, 0 },
-    { "psi", "kg/(m*s^2)", 689475729317, 1, -8 },
-    { "torr", "kg/(m*s^2)", 133322368421, 1, -9 },
-    { "mmHg", "kg/(m*s^2)", 133322368421, 1, -9 },
-    { "inHg", "kg/(m*s^2)", 338638815789, 1, -8 },
-    { "inH2O", "kg/(m*s^2)", 24884, 1, -2 },
+    { "N", "kg*m/s\0362", 1, 1, 0 },
+    { "dyn", "kg*m/s\0362", 1, 1, -5 },
+    { "gf", "kg*m/s\0362", 980665, 1, -8 },
+    { "kip", "kg*m/s\0362", 444822161526, 1, -8 },
+    { "lbf", "kg*m/s\0362", 444822161526, 1, -11 },
+    { "pdl", "kg*m/s\0362", 138254954376, 1, -12 },
+    { "J", "kg*m\0362/s\0362", 1, 1, 0 },
+    { "erg", "kg*m\0362/s\0362", 1, 1, -7 },
+    { "kcal", "kg*m\0362/s\0362", 41868, 1, -1 },
+    { "cal", "kg*m\0362/s\0362", 41868, 1, -4 },
+    { "Btu", "kg*m\0362/s\0362", 105505585262, 1, -8 },
+    { "ft*lbf", "kg*m\0362/s\0362", 135581794833, 1, -11 },
+    { "therm", "kg*m\0362/s\0362", 105506, 1, 3 },
+    { "MeV", "kg*m\0362/s\0362", 160217733, 1, -21 },
+    { "eV", "kg*m\0362/s\0362", 160217733, 1, -27 },
+    { "W", "kg*m\0362/s\0363", 1, 1, 0 },
+    { "hp", "kg*m\0362/s\0363", 745699871582, 1, -9 },
+    { "Pa", "kg/(m*s\0362)", 1, 1, 0 },
+    { "atm", "kg/(m*s\0362)", 101325, 1, 0 },
+    { "bar", "kg/(m*s\0362)", 100000, 1, 0 },
+    { "psi", "kg/(m*s\0362)", 689475729317, 1, -8 },
+    { "torr", "kg/(m*s\0362)", 133322368421, 1, -9 },
+    { "mmHg", "kg/(m*s\0362)", 133322368421, 1, -9 },
+    { "inHg", "kg/(m*s\0362)", 338638815789, 1, -8 },
+    { "inH2O", "kg/(m*s\0362)", 24884, 1, -2 },
     { "\23C", "K", 1, 1, 0 }, // offset = 27315 / 100
     { "\23F", "K", 5, 9, 0 }, // offset = 229835 / 900
     { "K", NULL, 1, 1, 0 },
     { "\23R", "K", 5, 9, 0 },
-    { "V", "kg*m^2/(A*s^3)", 1, 1, 0 },
+    { "V", "kg*m\0362/(A*s\0363)", 1, 1, 0 },
     { "A", NULL, 1, 1, 0 },
     { "C", "A*s", 1, 1, 0 },
-    { "\202", "kg*m^2/(A^2*s^3)", 1, 1, 0 },
-    { "F", "A^2*s^4/(kg*m^2)", 1, 1, 0 },
-    { "W", "kg*m^2/s^3", 1, 1, 0 },
+    { "\202", "kg*m\0362/(A\0362*s\0363)", 1, 1, 0 },
+    { "F", "A\0362*s\0364/(kg*m\0362)", 1, 1, 0 },
+    { "W", "kg*m\0362/s\0363", 1, 1, 0 },
     { "Fdy", "A*s", 96487, 1, 0 },
-    { "H", "kg*m^2/(A^2*s^2)", 1, 1, 0 },
-    { "mho", "A^2*s^3/(kg*m^2)", 1, 1, 0 },
-    { "S", "A^2*s^3/(kg*m^2)", 1, 1, 0 },
-    { "T", "kg/(A*s^2)", 1, 1, 0 },
-    { "Wb", "kg*m^2/(A*s^2)", 1, 1, 0 },
+    { "H", "kg*m\0362/(A\0362*s\0362)", 1, 1, 0 },
+    { "mho", "A\0362*s\0363/(kg*m\0362)", 1, 1, 0 },
+    { "S", "A\0362*s\0363/(kg*m\0362)", 1, 1, 0 },
+    { "T", "kg/(A*s\0362)", 1, 1, 0 },
+    { "Wb", "kg*m\0362/(A*s\0362)", 1, 1, 0 },
     { "\23", "r", 0, 18, -1 }, // pi
     { "r", NULL, 1, 1, 0 },
     { "grad", "r", 0, 2, -2 }, // pi
     { "arcmin", "r", 0, 108, -2 }, // pi
     { "arcs", "r", 0, 648, -3 }, // pi
     { "sr", NULL, 1, 1, 0 },
-    { "fc", "cd*sr/m^2", 107639104167, 1, -10 },
-    { "flam", "cd/m^2", 342625909964, 1, -11 },
-    { "lx", "cd*sr/m^2", 1, 1, 0 },
-    { "ph", "cd*sr/m^2", 1, 1, 4 },
-    { "sb", "cd/m^2", 1, 1, 4 },
+    { "fc", "cd*sr/m\0362", 107639104167, 1, -10 },
+    { "flam", "cd/m\0362", 342625909964, 1, -11 },
+    { "lx", "cd*sr/m\0362", 1, 1, 0 },
+    { "ph", "cd*sr/m\0362", 1, 1, 4 },
+    { "sb", "cd/m\0362", 1, 1, 4 },
     { "lm", "cd*sr", 1, 1, 0 },
     { "cd", NULL, 1, 1, 0 },
-    { "lam", "cd/m^2", 1, 0, 4 }, // pi
-    { "Gy", "m^2/s^2", 1, 1, 0 },
-    { "rad", "m^2/s^2", 1, 1, -2 },
-    { "rem", "m^2/s^2", 1, 1, -2 },
-    { "Sv", "m^2/s^2", 1, 1, 0 },
+    { "lam", "cd/m\0362", 1, 0, 4 }, // pi
+    { "Gy", "m\0362/s\0362", 1, 1, 0 },
+    { "rad", "m\0362/s\0362", 1, 1, -2 },
+    { "rem", "m\0362/s\0362", 1, 1, -2 },
+    { "Sv", "m\0362/s\0362", 1, 1, 0 },
     { "Bq", "1/s", 1, 1, 0 },
     { "Ci", "1/s", 37, 1, 9 },
     { "R", "A*s/kg", 258, 1, -6 },
     { "P", "kg/(m*s)", 1, 1, -1 },
-    { "St", "m^2/s", 1, 1, -4 },
+    { "St", "m\0362/s", 1, 1, -4 },
     { "one", "1", 1, 1, 0 },
     { NULL, NULL, 0, 0, 0 }
 };
@@ -789,7 +828,7 @@ static const unitdef *find_unit(std::string s, int *exponent, vartype **user, st
             idx++;
         }
         // Not in units table; look for user-defined unit...
-        vartype *v = recall_var(s.c_str(), s.length());
+        vartype *v = recall_var(s.c_str(), (int) s.length());
         if (v != NULL && (v->type == TYPE_REAL || v->type == TYPE_UNIT)) {
             *user = v;
             *un = s;
@@ -810,6 +849,8 @@ static const unitdef *find_unit(std::string s, int *exponent, vartype **user, st
         }
         char prefix = s[0];
         switch (prefix) {
+            case 'Q': *exponent = 30; break;
+            case 'R': *exponent = 27; break;
             case 'Y': *exponent = 24; break;
             case 'Z': *exponent = 21; break;
             case 'E': *exponent = 18; break;
@@ -829,6 +870,8 @@ static const unitdef *find_unit(std::string s, int *exponent, vartype **user, st
             case 'a': *exponent = -18; break;
             case 'z': *exponent = -21; break;
             case 'y': *exponent = -24; break;
+            case 'r': *exponent = -27; break;
+            case 'q': *exponent = -30; break;
             default: *user = NULL; return NULL;
         }
         s = s.substr(1, s.length() - 1);
@@ -838,10 +881,25 @@ static const unitdef *find_unit(std::string s, int *exponent, vartype **user, st
 /* Hack to get things to compile in Ubuntu 14.04 */
 static std::string to_string(int i) {
     char buf[20];
-    sprintf(buf, "%d", i);
+    snprintf(buf, 20, "%d", i);
     return std::string(buf, strlen(buf));
 }
 
+struct UnitLink {
+    std::string name;
+    UnitLink *next;
+    UnitLink(std::string name, UnitLink *next) : name(name), next(next) {}
+    bool circular() {
+        UnitLink *ptr = next;
+        while (ptr != NULL) {
+            if (name == ptr->name)
+                return true;
+            ptr = ptr->next;
+        }
+        return false;
+    }
+};
+
 struct UnitProduct {
     std::map<std::string, int> elem;
     UnitProduct() {}
@@ -882,14 +940,14 @@ struct UnitProduct {
             else if (iter->second == 1)
                 numer = numer + "*" + iter->first;
             else if (iter->second > 1)
-                numer = numer + "*" + iter->first + "^" + to_string(iter->second);
+                numer = numer + "*" + iter->first + "\36" + to_string(iter->second);
             else {
                 if (denom != "")
                     compoundDenom = true;
                 if (iter->second == -1)
                     denom = denom + "*" + iter->first;
                 else
-                    denom = denom + "*" + iter->first + "^" + to_string(-iter->second);
+                    denom = denom + "*" + iter->first + "\36" + to_string(-iter->second);
             }
         }
         if (numer != "")
@@ -907,7 +965,7 @@ struct UnitProduct {
         }
     }
 
-    bool toBase(phloat *f, std::string *s);
+    bool toBase(phloat *f, std::string *s, UnitLink *link = NULL);
 };
 
 class UnitLexer {
@@ -946,7 +1004,7 @@ class UnitLexer {
     }
 
     bool isIdentifier(std::string s) {
-        int len = s.length();
+        int len = (int) s.length();
         if (len == 0 || !isIdentifierStartChar(s[0]))
             return false;
         for (int i = 1; i < len; i++)
@@ -1151,7 +1209,7 @@ class UnitParser {
     }
 };
 
-bool UnitProduct::toBase(phloat *f, std::string *s) {
+bool UnitProduct::toBase(phloat *f, std::string *s, UnitLink *link) {
     phloat v = 1;
     int exp = 0;
     std::string us = "";
@@ -1174,8 +1232,12 @@ bool UnitProduct::toBase(phloat *f, std::string *s) {
                     s2 = userName;
                 } else {
                     int errpos;
-                    UnitProduct *up = UnitParser::parse(std::string(u->text, u->length), &errpos);
-                    bool success = up->toBase(&f2, &s2);
+                    std::string un(u->text, u->length);
+                    UnitLink linque(un, link);
+                    if (linque.circular())
+                        return false;
+                    UnitProduct *up = UnitParser::parse(un, &errpos);
+                    bool success = up->toBase(&f2, &s2, &linque);
                     delete up;
                     if (!success)
                         return false;
@@ -1188,7 +1250,7 @@ bool UnitProduct::toBase(phloat *f, std::string *s) {
                 else {
                     us += "(";
                     us += s2;
-                    us += ")^";
+                    us += ")\36";
                     us += to_string(p);
                 }
             } else {
@@ -1204,7 +1266,7 @@ bool UnitProduct::toBase(phloat *f, std::string *s) {
             else {
                 us += "(";
                 us += un;
-                us += ")^";
+                us += ")\36";
                 us += to_string(p);
             }
             exp += (ud->exp + e) * p;
@@ -1245,6 +1307,28 @@ bool is_unit(const char *text, int length) {
     return success;
 }
 
+bool is_custom_menu_unit(const char *text, int length) {
+    // We want to avoid treating things like units that are really
+    // just undefined labels. We do this by making sure that the
+    // text parses as a unit, *and* that all the components of that
+    // unit are either built-in units, or existing unit variables.
+    int errpos;
+    UnitProduct *up = UnitParser::parse(std::string(text, length), &errpos);
+    if (up == NULL)
+        return false;
+    int exp;
+    vartype *user;
+    std::string un;
+    for (std::map<std::string, int>::iterator iter = up->elem.begin(); iter != up->elem.end(); iter++) {
+        if (find_unit(iter->first, &exp, &user, &un) == NULL && user == NULL) {
+            delete up;
+            return false;
+        }
+    }
+    delete up;
+    return true;
+}
+
 static int get_value_and_base(const vartype *v, phloat *value, std::string *baseUnit, phloat *factor) {
     if (v->type == TYPE_REAL) {
         *value = ((vartype_real *) v)->x;
@@ -1267,6 +1351,17 @@ static bool equiv_units(const std::string &x, const std::string &y) {
     return x == y || x == "" && y == "r" || x == "r" && y == "";
 }
 
+#ifdef BCD_MATH
+    /* Exact decimal temperature conversion constants */
+    const Phloat LIT_1_8(18, 10);
+    const Phloat LIT_273_15(27315, 100);
+    const Phloat LIT_459_67(45967, 100);
+#else
+    #define LIT_1_8 1.8
+    #define LIT_273_15 273.15
+    #define LIT_459_67 459.67
+#endif
+
 int convert_helper(const vartype *X, const vartype *Y, phloat *res) {
     /* Convert Y to match the units of X; used for CONVERT,
      * addition, and subtraction
@@ -1292,32 +1387,32 @@ int convert_helper(const vartype *X, const vartype *Y, phloat *res) {
             // only be another temperature unit
             if (bux == "\23C")
                 if (buy == "\23F")
-                    *res = (y - 32) / 1.8;
+                    *res = (y - 32) / LIT_1_8;
                 else if (buy == "K")
-                    *res = y - 273.15;
+                    *res = y - LIT_273_15;
                 else // buy == "\23R"
-                    *res = y / 1.8 - 273.15;
+                    *res = y / LIT_1_8 - LIT_273_15;
             else if (bux == "\23F")
                 if (buy == "\23C")
-                    *res = y * 1.8 + 32;
+                    *res = y * LIT_1_8 + 32;
                 else if (buy == "K")
-                    *res = y * 1.8 - 459.67;
+                    *res = y * LIT_1_8 - LIT_459_67;
                 else // buy == "\23R"
-                    *res = y - 459.67;
+                    *res = y - LIT_459_67;
             else if (bux == "K")
                 if (buy == "\23C")
-                    *res = y + 273.15;
+                    *res = y + LIT_273_15;
                 else if (buy == "\23F")
-                    *res = (y + 459.67) / 1.8;
+                    *res = (y + LIT_459_67) / LIT_1_8;
                 else // buy == "\23R" (can't get here)
-                    *res = y / 1.8;
+                    *res = y / LIT_1_8;
             else // bux == "\23R"
                 if (buy == "\23C")
-                    *res = (y + 273.15) * 1.8;
+                    *res = (y + LIT_273_15) * LIT_1_8;
                 else if (buy == "\23F")
-                    *res = y + 459.67;
+                    *res = y + LIT_459_67;
                 else // buy == "K" (can't get here)
-                    *res = y * 1.8;
+                    *res = y * LIT_1_8;
         } else
             goto normal;
     } else {
@@ -1397,9 +1492,9 @@ int docmd_ubase(arg_struct *arg) {
         vartype_unit *u = (vartype_unit *) stack[sp];
         std::string ou(u->text, u->length);
         if (ou == "\23C")
-            r = x + 273.15;
+            r = x + LIT_273_15;
         else if (ou == "\23F")
-            r = (x + 459.67) / 1.8;
+            r = (x + LIT_459_67) / LIT_1_8;
         else
             goto normal;
     } else {
@@ -1414,7 +1509,7 @@ int docmd_ubase(arg_struct *arg) {
         else
             return ERR_OUT_OF_RANGE;
     }
-    vartype *res = new_unit(r, bu.c_str(), bu.length());
+    vartype *res = new_unit(r, bu.c_str(), (int) bu.length());
     if (res == NULL)
         return ERR_INSUFFICIENT_MEMORY;
     unary_result(res);
@@ -1486,7 +1581,7 @@ int docmd_ufact(arg_struct *arg) {
         else
             return ERR_OUT_OF_RANGE;
     }
-    vartype *res = new_unit(r, newUnit.c_str(), newUnit.length());
+    vartype *res = new_unit(r, newUnit.c_str(), (int) newUnit.length());
     if (res == NULL)
         return ERR_INSUFFICIENT_MEMORY;
     return binary_result(res);
@@ -1642,7 +1737,7 @@ static int unit_mul_div(const vartype *x, const vartype *y, vartype **r, int mod
         else
             return ERR_OUT_OF_RANGE;
     }
-    *r = new_unit(res, nru.c_str(), nru.length());
+    *r = new_unit(res, nru.c_str(), (int) nru.length());
     if (*r == NULL)
         return ERR_INSUFFICIENT_MEMORY;
     return ERR_NONE;
@@ -1673,7 +1768,7 @@ int unit_pow(vartype *x, phloat e, vartype **r) {
         up->pow(ie);
         goto finish;
     } else {
-        ie = to_int4(1 / e + 0.5);
+        ie = to_int4(e > 0 ? 1 / e + 0.5 : 1 / e - 0.5);
         e1 = 1 / ((phloat) ie);
         if (e1 != e && e1 != nextafter(e, POS_HUGE_PHLOAT) && e1 != nextafter(e, NEG_HUGE_PHLOAT))
             // Not close enough to the reciprocal of an integer
@@ -1699,7 +1794,7 @@ int unit_pow(vartype *x, phloat e, vartype **r) {
             else
                 return ERR_OUT_OF_RANGE;
         }
-        *r = new_unit(res, nu.c_str(), nu.length());
+        *r = new_unit(res, nu.c_str(), (int) nu.length());
         return *r == NULL ? ERR_INSUFFICIENT_MEMORY : ERR_NONE;
     }
 }
