@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Plus42 -- an enhanced HP-42S calculator simulator
- * Copyright (C) 2004-2022  Thomas Okken
+ * Copyright (C) 2004-2025  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -38,8 +38,11 @@
 #define PLOT_RESULT_NONE 0
 #define PLOT_RESULT_EVAL 1
 #define PLOT_RESULT_SOLVE 2
-#define PLOT_RESULT_SOLVE_FAIL 3
-#define PLOT_RESULT_INTEG 4
+#define PLOT_RESULT_SOLVE_DIRECT 3
+#define PLOT_RESULT_SOLVE_SIGN_REVERSAL 4
+#define PLOT_RESULT_SOLVE_EXTREMUM 5
+#define PLOT_RESULT_SOLVE_FAIL 6
+#define PLOT_RESULT_INTEG 7
 
 #define PLOT_FUN 0
 #define PLOT_X_VAR 1
@@ -346,7 +349,7 @@ static void validate_axes(PlotData *data) {
         bool found = false;
         for (int j = 0; j < params.size(); j++) {
             std::string s = params[j];
-            if (string_equals(data->axes[i].name, data->axes[i].len, s.c_str(), s.length())) {
+            if (string_equals(data->axes[i].name, data->axes[i].len, s.c_str(), (int) s.length())) {
                 found = true;
                 break;
             }
@@ -720,12 +723,12 @@ static int prepare_plot(PlotData *data) {
         bool x_found = false, y_found = data->axes[1].len == 0;
         for (int i = 0; i < params.size(); i++) {
             std::string s = params[i];
-            if (!x_found && string_equals(data->axes[0].name, data->axes[0].len, s.c_str(), s.length())) {
+            if (!x_found && string_equals(data->axes[0].name, data->axes[0].len, s.c_str(), (int) s.length())) {
                 x_found = true;
                 if (y_found)
                     break;
             }
-            if (!y_found && string_equals(data->axes[1].name, data->axes[1].len, s.c_str(), s.length())) {
+            if (!y_found && string_equals(data->axes[1].name, data->axes[1].len, s.c_str(), (int) s.length())) {
                 y_found = true;
                 if (x_found)
                     break;
@@ -900,7 +903,7 @@ void display_plot_params(int key) {
             if (err == ERR_NONE)
                 draw_message(0, "Variables Printed", 17);
             else {
-                display_error(err, false);
+                display_error(err);
                 flush_display();
             }
             return;
@@ -922,7 +925,7 @@ static vartype *integ_result_unit(PlotData *data) {
             std::string r;
             if (!normalize_unit(std::string(ux->text, ux->length) + "*" + std::string(uy->text, uy->length), &r))
                 return NULL;
-            return new_unit(0, r.c_str(), r.length());
+            return new_unit(0, r.c_str(), (int) r.length());
         } else {
             return dup_vartype((vartype *) ux);
         }
@@ -968,12 +971,36 @@ int return_to_plot(bool failure, bool stop) {
     phloat ymax = data.axes[1].max;
 
     if (!failure && sp != -1 && (res->type == TYPE_REAL || res->type == TYPE_UNIT)) {
-        if ((state == PLOT_STATE_SOLVE || state != PLOT_STATE_INTEG && data.axes[1].len > 0) && stack[sp - 1]->type != TYPE_STRING && ((vartype_real *) stack[sp - 3])->x != 0) {
+        phloat errp;
+        if ((state == PLOT_STATE_SOLVE || state != PLOT_STATE_INTEG && data.axes[1].len > 0)
+                && stack[sp - 1]->type != TYPE_STRING
+                && (errp = ((vartype_real *) stack[sp - 3])->x) != SOLVE_ROOT) {
             // Not an error, but the solver didn't find a root
             if (state == PLOT_STATE_SOLVE) {
                 replot = true;
-                data.set_phloat(PLOT_RESULT, data.result = ((vartype_real *) stack[sp - 3])->x);
-                data.set_int(PLOT_RESULT_TYPE, data.result_type = PLOT_RESULT_SOLVE_FAIL);
+                if (errp != SOLVE_SIGN_REVERSAL && errp != SOLVE_EXTREMUM) {
+                    // Bad Guess(es) or Constant? -- actual failures to find anything
+                    data.set_phloat(PLOT_RESULT, data.result = errp);
+                    data.set_int(PLOT_RESULT_TYPE, data.result_type = PLOT_RESULT_SOLVE_FAIL);
+                } else {
+                    // Sign Reversal or Extremum -- we want to return the actual point
+                    // the solver ended up converging on
+                    phloat x;
+                    int err = convert_helper(data.axes[0].unit, res, &x);
+                    if (err != ERR_NONE)
+                        return err;
+                    data.set_phloat(PLOT_RESULT, data.result = x);
+                    if (errp == SOLVE_SIGN_REVERSAL)
+                        data.set_int(PLOT_RESULT_TYPE, data.result_type = PLOT_RESULT_SOLVE_SIGN_REVERSAL);
+                    else
+                        data.set_int(PLOT_RESULT_TYPE, data.result_type = PLOT_RESULT_SOLVE_EXTREMUM);
+                    if (data.axes[0].unit->type == TYPE_UNIT) {
+                        vartype_unit *u = (vartype_unit *) data.axes[0].unit;
+                        result = new_unit(x, u->text, u->length);
+                    } else {
+                        result = new_real(x);
+                    }
+                }
             }
             goto fail;
         }
@@ -985,9 +1012,12 @@ int return_to_plot(bool failure, bool stop) {
             if (u == NULL)
                 return ERR_INVALID_UNIT;
             int err = convert_helper(u, res, &y);
-            free_vartype(u);
-            if (err != ERR_NONE)
+            if (err != ERR_NONE) {
+                free_vartype(u);
                 return err;
+            }
+            ((vartype_real *) u)->x = y;
+            result = u;
         } else if (state == PLOT_STATE_SOLVE) {
             // And, of course, in the case of SOLVE, the result will have X axis units
             int err = convert_helper(data.axes[0].unit, res, &y);
@@ -1112,13 +1142,19 @@ int return_to_plot(bool failure, bool stop) {
         } else if (state == PLOT_STATE_SOLVE) {
             replot = true;
             data.set_phloat(PLOT_RESULT, data.result = y);
-            data.set_int(PLOT_RESULT_TYPE, data.result_type = PLOT_RESULT_SOLVE);
-            result = new_real(y);
+            data.set_int(PLOT_RESULT_TYPE, data.result_type =
+                    stack[sp - 1]->type == TYPE_STRING ? PLOT_RESULT_SOLVE_DIRECT : PLOT_RESULT_SOLVE);
+            if (data.axes[0].unit->type == TYPE_UNIT) {
+                vartype_unit *u = (vartype_unit *) data.axes[0].unit;
+                result = new_unit(y, u->text, u->length);
+            } else {
+                result = new_real(y);
+            }
         } else if (state == PLOT_STATE_INTEG) {
             replot = true;
             data.set_phloat(PLOT_RESULT, data.result = y);
             data.set_int(PLOT_RESULT_TYPE, data.result_type = PLOT_RESULT_INTEG);
-            result = new_real(y);
+            // already set result earlier
         }
     } else {
         fail:
@@ -1172,7 +1208,10 @@ int return_to_plot(bool failure, bool stop) {
                             result_unit = data.axes[1].unit;
                             break;
                         }
-                        case PLOT_RESULT_SOLVE: {
+                        case PLOT_RESULT_SOLVE:
+                        case PLOT_RESULT_SOLVE_DIRECT:
+                        case PLOT_RESULT_SOLVE_SIGN_REVERSAL:
+                        case PLOT_RESULT_SOLVE_EXTREMUM: {
                             if (data.axes[0].len == 0)
                                 string2buf(buf, 100, &pos, "<X>", 3);
                             else
@@ -1209,6 +1248,32 @@ int return_to_plot(bool failure, bool stop) {
                             if (data.result_type == PLOT_RESULT_INTEG)
                                 free_vartype(result_unit);
                         }
+                        if (data.result_type == PLOT_RESULT_SOLVE_DIRECT
+                                || data.result_type == PLOT_RESULT_SOLVE_SIGN_REVERSAL
+                                || data.result_type == PLOT_RESULT_SOLVE_EXTREMUM) {
+                            const char *text;
+                            int len;
+                            switch (to_int(data.result_type)) {
+                                case PLOT_RESULT_SOLVE_DIRECT: {
+                                    vartype_string *s = (vartype_string *) stack[sp - 1];
+                                    text = s->txt();
+                                    len = s->length;
+                                    break;
+                                }
+                                case PLOT_RESULT_SOLVE_SIGN_REVERSAL: {
+                                    text = solve_message[SOLVE_SIGN_REVERSAL].text;
+                                    len = solve_message[SOLVE_SIGN_REVERSAL].length;
+                                    break;
+                                }
+                                case PLOT_RESULT_SOLVE_EXTREMUM: {
+                                    text = solve_message[SOLVE_EXTREMUM].text;
+                                    len = solve_message[SOLVE_EXTREMUM].length;
+                                    break;
+                                }
+                            }
+                            char2buf(buf, 100, &pos, ' ');
+                            string2buf(buf, 100, &pos, text, len);
+                        }
                     }
                     int w = small_string_width(buf, pos);
                     if (w > mode_plot_result_width)
@@ -1463,7 +1528,7 @@ static bool run_plot(bool reset) {
     mode_plot_viewer = false;
     int err = plot_helper(reset);
     if (err != ERR_NONE && err != ERR_RUN) {
-        display_error(err, false);
+        display_error(err);
         flush_display();
         return false;
     }
