@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Plus42 -- an enhanced HP-42S calculator simulator
- * Copyright (C) 2004-2022  Thomas Okken
+ * Copyright (C) 2004-2025  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -23,9 +23,12 @@
 
 bool unpersist_eqn(int4 ver);
 bool persist_eqn();
+void reset_eqn();
 
 int eqn_start(int whence);
 void eqn_end();
+int eqn_new();
+int eqn_edit();
 
 bool eqn_active();
 bool eqn_alt_keys();
@@ -40,7 +43,9 @@ int eqn_repeat();
 bool eqn_timeout();
 void eqn_restart_cursor();
 
-int return_to_eqn_edit();
+int return_to_eqn_edit(int error);
+void eqn_save_error_pos(int eqn_id, int pos);
+void eqn_display_error(bool temp);
 
 
 #endif
