@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Plus42 -- an enhanced HP-42S calculator simulator
- * Copyright (C) 2004-2022  Thomas Okken
+ * Copyright (C) 2004-2025  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -85,6 +85,8 @@ extern FILE *gfile;
 #define ERR_VARIABLE_EXISTS        49
 #define ERR_TOO_MANY_ARGUMENTS     50
 #define ERR_NO_SOLUTION_FOUND      51
+#define ERR_PROGRAM_LOCKED         52
+#define ERR_NEXT_PROGRAM_LOCKED    53
 
 #define RTNERR_MAX 8
 
@@ -162,75 +164,87 @@ extern const error_spec errors[];
 #define MENU_ALPHA_MATH2   15
 #define MENU_ALPHA_PUNC1   16
 #define MENU_ALPHA_PUNC2   17
-#define MENU_ALPHA_MISC1   18
-#define MENU_ALPHA_MISC2   19
-#define MENU_ST            20
-#define MENU_IND_ST        21
-#define MENU_IND           22
-#define MENU_MODES1        23
-#define MENU_MODES2        24
-#define MENU_MODES3        25
-#define MENU_MODES4        26
-#define MENU_MODES5        27
-#define MENU_DISP1         28
-#define MENU_DISP2         29
-#define MENU_DISP3         30
-#define MENU_CLEAR1        31
-#define MENU_CLEAR2        32
-#define MENU_CONVERT1      33
-#define MENU_CONVERT2      34
-#define MENU_FLAGS         35
-#define MENU_PROB          36
-#define MENU_CUSTOM1       37
-#define MENU_CUSTOM2       38
-#define MENU_CUSTOM3       39
-#define MENU_PGM_FCN1      40
-#define MENU_PGM_FCN2      41
-#define MENU_PGM_FCN3      42
-#define MENU_PGM_FCN4      43
-#define MENU_PGM_XCOMP0    44
-#define MENU_PGM_XCOMPY    45
-#define MENU_PRINT1        46
-#define MENU_PRINT2        47
-#define MENU_PRINT3        48
-#define MENU_TOP_FCN       49
-#define MENU_CATALOG       50
-#define MENU_BLANK         51
-#define MENU_PROGRAMMABLE  52
-#define MENU_VARMENU       53
-#define MENU_STAT1         54
-#define MENU_STAT2         55
-#define MENU_STAT_CFIT     56
-#define MENU_STAT_MODL     57
-#define MENU_STAT_SUMS1    58
-#define MENU_STAT_SUMS2    59
-#define MENU_STAT_SUMS3    60
-#define MENU_MATRIX1       61
-#define MENU_MATRIX2       62
-#define MENU_MATRIX3       63
-#define MENU_MATRIX_SIMQ   64
-#define MENU_MATRIX_EDIT1  65
-#define MENU_MATRIX_EDIT2  66
-#define MENU_BASE          67
-#define MENU_BASE_A_THRU_F 68
-#define MENU_BASE_LOGIC    69
-#define MENU_SOLVE         70
-#define MENU_INTEG         71
-#define MENU_INTEG_PARAMS  72
-#define MENU_DIR_FCN1      73
-#define MENU_DIR_FCN2      74
-#define MENU_UNIT_FCN      75
-#define MENU_TVM_APP1      76
-#define MENU_TVM_APP2      77
-#define MENU_TVM_AMORT     78
-#define MENU_TVM_TABLE     79
-#define MENU_TVM_PARAMS    80
-#define MENU_TVM_PRGM1     81
-#define MENU_TVM_PRGM2     82
-#define MENU_EQN_FCN       83
-#define MENU_GRAPH         84
-#define MENU_GRAPH_AXES    85
-#define MENU_GRAPH_VIEW    86
+#define MENU_ALPHA_PUNC3   18
+#define MENU_ALPHA_MISC1   19
+#define MENU_ALPHA_MISC2   20
+#define MENU_ST            21
+#define MENU_IND_ST        22
+#define MENU_IND           23
+#define MENU_MODES1        24
+#define MENU_MODES2        25
+#define MENU_MODES3        26
+#define MENU_MODES4        27
+#define MENU_MODES_MENUS   28
+#define MENU_DISP1         29
+#define MENU_DISP2         30
+#define MENU_DISP3         31
+#define MENU_DISP4         32
+#define MENU_CLEAR1        33
+#define MENU_CLEAR2        34
+#define MENU_CONVERT1      35
+#define MENU_CONVERT2      36
+#define MENU_FLAGS         37
+#define MENU_PROB          38
+#define MENU_CUSTOM1       39
+#define MENU_CUSTOM2       40
+#define MENU_CUSTOM3       41
+#define MENU_PGM_FCN1      42
+#define MENU_PGM_FCN2      43
+#define MENU_PGM_FCN3      44
+#define MENU_PGM_FCN4      45
+#define MENU_PGM_XCOMP0    46
+#define MENU_PGM_XCOMPY    47
+#define MENU_PRINT1        48
+#define MENU_PRINT2        49
+#define MENU_PRINT3        50
+#define MENU_TOP_FCN       51
+#define MENU_CATALOG       52
+#define MENU_BLANK         53
+#define MENU_PROGRAMMABLE  54
+#define MENU_VARMENU       55
+#define MENU_STAT1         56
+#define MENU_STAT2         57
+#define MENU_STAT_CFIT     58
+#define MENU_STAT_MODL     59
+#define MENU_STAT_SUMS1    60
+#define MENU_STAT_SUMS2    61
+#define MENU_STAT_SUMS3    62
+#define MENU_MATRIX1       63
+#define MENU_MATRIX2       64
+#define MENU_MATRIX3       65
+#define MENU_MATRIX_SIMQ   66
+#define MENU_MATRIX_EDIT1  67
+#define MENU_MATRIX_EDIT2  68
+#define MENU_BASE1         69
+#define MENU_BASE2         70
+#define MENU_BASE3         71
+#define MENU_BASE4         72
+#define MENU_BASE5         73
+#define MENU_BASE_A_THRU_F 74
+#define MENU_BASE_LOGIC    75
+#define MENU_BASE_FLOAT1   76
+#define MENU_BASE_FLOAT2   77
+#define MENU_BASE_MODES    78
+#define MENU_BASE_DISP     79
+#define MENU_SOLVE         80
+#define MENU_INTEG         81
+#define MENU_INTEG_PARAMS  82
+#define MENU_DIR_FCN1      83
+#define MENU_DIR_FCN2      84
+#define MENU_UNIT_FCN1     85
+#define MENU_UNIT_FCN2     86
+#define MENU_TVM_APP1      87
+#define MENU_TVM_APP2      88
+#define MENU_TVM_AMORT     89
+#define MENU_TVM_TABLE     90
+#define MENU_TVM_PARAMS    91
+#define MENU_TVM_PRGM1     92
+#define MENU_TVM_PRGM2     93
+#define MENU_EQN_FCN1      94
+#define MENU_EQN_FCN2      95
+#define MENU_GRAPH         96
+#define MENU_GRAPH_AXES    97
+#define MENU_GRAPH_VIEW    98
 
 
 struct menu_item_spec {
@@ -255,6 +269,7 @@ extern const menu_spec menus[];
 
 /* Suppress menu updates while state loading is in progress */
 extern bool loading_state;
+extern bool saving_state;
 
 
 /* Registers */
@@ -377,9 +392,13 @@ struct var_struct {
 struct prgm_struct {
     int4 capacity;
     int4 size;
-    int lclbl_invalid;
+    bool lclbl_invalid;
+    bool locked;
     unsigned char *text;
     equation_data *eq_data;
+    inline bool is_end(int4 pc) {
+        return text[pc] == CMD_END && (text[pc + 1] & 112) == 0;
+    }
 };
 struct label_struct {
     unsigned char length;
@@ -479,6 +498,17 @@ extern int mode_goose;
 extern bool mode_time_clktd;
 extern bool mode_time_clk24;
 extern int mode_wsize;
+extern bool mode_carry;
+extern bool mode_dec_int;
+extern bool mode_bin_sep;
+extern bool mode_oct_sep;
+extern bool mode_dec_sep;
+extern bool mode_hex_sep;
+extern bool mode_menu_caps;
+extern bool mode_menu_static;
+#if defined(ANDROID) || defined(IPHONE)
+extern bool mode_popup_unknown;
+#endif
 extern bool mode_header;
 extern int mode_amort_seq;
 extern bool mode_plot_viewer;
@@ -486,6 +516,12 @@ extern int mode_plot_key;
 extern int mode_plot_sp;
 extern vartype *mode_plot_inv;
 extern int mode_plot_result_width;
+extern bool mode_multi_line;
+extern bool mode_lastx_top;
+extern bool mode_alpha_top;
+extern bool mode_header_flags;
+extern bool mode_header_polar;
+extern bool mode_matedit_stk;
 
 extern phloat entered_number;
 extern int entered_string_length;
@@ -565,6 +601,9 @@ extern int4 incomplete_saved_highlight_row;
 #define CATSECT_DIRS_ONLY 54
 #define CATSECT_OTHER 55
 #define CATSECT_LIST_STR_ONLY 56
+#define CATSECT_MAT_LIST 57
+#define CATSECT_MAT_LIST_ONLY 58
+#define CATSECT_LIST_ONLY 59
 
 /* Command line handling temporaries */
 extern char cmdline[100];
@@ -581,6 +620,39 @@ extern int4 matedit_i;
 extern int4 matedit_j;
 extern int matedit_prev_appmenu;
 
+struct matedit_stack_entry {
+    int4 coord;
+    int4 anchor;
+    void set(int4 coord, int4 anchor) {
+        this->coord = coord;
+        this->anchor = anchor;
+    }
+    void set(int8 combined) {
+        coord = (int4) combined;
+        anchor = (int4) (combined >> 32);
+    }
+    int8 as_int8() {
+        return coord & 0x0ffffffffLL | (((int8) anchor) << 32);
+    }
+    void set(phloat combined) {
+        int8 c = to_int8(combined);
+        coord = (int4) c;
+        int2 offset = (int2) (c >> 32);
+        anchor = coord + offset;
+    }
+    phloat as_phloat() {
+        int4 offset = anchor - coord;
+        int8 combined = coord & 0x0ffffffffLL | ((offset & 0x0ffffLL) << 32);
+        return phloat(combined);
+    }
+};
+
+extern matedit_stack_entry *matedit_stack;
+extern int matedit_stack_depth;
+extern bool matedit_is_list;
+extern int4 matedit_view_i;
+extern int4 matedit_view_j;
+
 /* INPUT */
 extern char input_name[11];
 extern int input_length;
@@ -623,43 +695,49 @@ extern bool no_keystrokes_yet;
 int4 new_eqn_idx();
 void clear_rtns_vars_and_prgms();
 int clear_prgm(const arg_struct *arg);
-int clear_prgm_by_index(pgm_index prgm);
 int clear_prgm_by_int_index(int prgm);
 void clear_prgm_lines(int4 count);
 void goto_dot_dot(bool force_new);
-int mvar_prgms_exist();
-int label_has_mvar(int lblindex);
+bool mvar_prgms_exist();
+bool label_has_mvar(int4 dir_id, int lblindex);
 int get_command_length(pgm_index prgm, int4 pc);
 void get_next_command(int4 *pc, int *command, arg_struct *arg, int find_target, const char **num_str);
 void rebuild_label_table();
+void count_embed_references(directory *dir, int prgm, bool up);
 void delete_command(int4 pc);
+int eqn_flip(int4 pc);
 bool store_command(int4 pc, int command, arg_struct *arg, const char *num_str);
 void store_command_after(int4 *pc, int command, arg_struct *arg, const char *num_str);
 int x2line();
 int a2line(bool append);
+int prgm_lock(bool lock);
 int4 pc2line(int4 pc);
 int4 line2pc(int4 line);
 int4 global_pc2line(pgm_index prgm, int4 pc);
 int4 global_line2pc(pgm_index prgm, int4 line);
 int4 find_local_label(const arg_struct *arg);
-bool find_global_label(const arg_struct *arg, pgm_index *prgm, int4 *pc);
-bool find_global_label_index(const arg_struct *arg, int *idx);
+bool find_global_label(const arg_struct *arg, pgm_index *prgm, int4 *pc, int *idx = NULL);
 int push_rtn_addr(pgm_index prgm, int4 pc);
 int push_indexed_matrix();
+void maybe_pop_indexed_matrix(const char *name, int len);
 int push_func_state(int n);
 int push_stack_state(bool big);
 int pop_func_state(bool error);
-int get_frame_depth(int *depth);
+int get_saved_stack_level(int level, vartype **res);
 void step_out();
 void step_over();
 void return_here_after_last_rtn();
-void unwind_after_eqn_error();
+void equation_deleted(int eqn_index);
+bool unwind_after_eqn_error();
 bool should_i_stop_at_this_level();
 int rtn(int err);
 int rtn_with_error(int err);
+bool need_fstart();
 void pop_rtn_addr(pgm_index *prgm, int4 *pc, bool *stop);
 void clear_all_rtns();
 int get_rtn_level();
+void save_csld();
+bool is_csld();
 bool solve_active();
 bool integ_active();
 bool solve_or_plot_active();
