@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Plus42 -- an enhanced HP-42S calculator simulator
- * Copyright (C) 2004-2022  Thomas Okken
+ * Copyright (C) 2004-2025  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -24,7 +24,6 @@
 #include "core_phloat.h"
 #include "core_helpers.h"
 #include "core_display.h"
-#include "shell.h"
 
 #ifndef BCD_MATH
 // We need these locally for BID128->double conversion
@@ -223,6 +222,20 @@ Phloat Phloat::operator=(Phloat p) {
     return *this;
 }
 
+/* public */
+void Phloat::assign17digits(double d) {
+    if (isinf(d) || isnan(d)) {
+        binary64_to_bid128(&val, &d);
+    } else {
+        char buf[25];
+        snprintf(buf, 25, "%.15e", d);
+        double d2;
+        if (sscanf(buf, "%le", &d2) != 1 || d != d2)
+            snprintf(buf, 25, "%.16e", d);
+        bid128_from_string(&val, buf);
+    }
+}
+
 /* public */
 bool Phloat::operator==(Phloat p) const {
     int r;
@@ -392,6 +405,12 @@ int p_isnan(Phloat p) {
     return r;
 }
 
+int p_isnormal(Phloat p) {
+    int r;
+    bid128_isNormal(&r, &p.val);
+    return r;
+}
+
 int to_digit(Phloat p) {
     BID_UINT128 ten, res;
     int d10 = 10;
@@ -740,6 +759,24 @@ Phloat nextafter(Phloat x, Phloat y) {
     return Phloat(res);
 }
 
+int ilogb(Phloat x) {
+    int res;
+    bid128_ilogb(&res, &x.val);
+    return res;
+}
+
+Phloat scalbn(Phloat x, int y) {
+    BID_UINT128 res;
+    bid128_scalbn(&res, &x.val, &y);
+    return Phloat(res);
+}
+
+Phloat copysign(Phloat x, Phloat y) {
+    BID_UINT128 res;
+    bid128_copySign(&res, &x.val, &y.val);
+    return Phloat(res);
+}
+
 Phloat operator*(int x, Phloat y) {
     BID_UINT128 xx, res;
     bid128_from_int32(&xx, &x);
@@ -824,7 +861,7 @@ int string2phloat(const char *buf, int buflen, phloat *d) {
      * 4: negative underflow
      * 5: other error
      */
-    char mantissa[16] = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
+    char mantissa[MAX_MANT_DIGITS];
     int mant_sign = 0;
     int skipping_zeroes = 1;
     int seen_dot = 0;
@@ -839,6 +876,8 @@ int string2phloat(const char *buf, int buflen, phloat *d) {
     int is_zero = 1;
     double res;
 
+    memset(mantissa, 0, MAX_MANT_DIGITS);
+
     for (i = 0; i < buflen; i++) {
         char c = buf[i];
         if (c == 24) {
@@ -870,7 +909,7 @@ int string2phloat(const char *buf, int buflen, phloat *d) {
             }
             /* Once we get here, c should be a digit */
             if (++mant_digits > MAX_MANT_DIGITS)
-                /* Too many digits! We only allow the user to enter 16 (binary) or 34 (decimal). */
+                /* Too many digits! We only allow the user to enter 17 (binary) or 34 (decimal). */
                 return 5;
             if (c == '0' && skipping_zeroes)
                 continue;
@@ -914,15 +953,15 @@ int string2phloat(const char *buf, int buflen, phloat *d) {
      * and 'mant_sign' contains the mantissa's sign.
      */
     char decstr[35];
-    char *cp = decstr;
+    int pos = 0;
     if (mant_sign)
-        *cp++ = '-';
-    for (i = 0; i < 16; i++) {
-        *cp++ = mantissa[i] + '0';
+        decstr[pos++] = '-';
+    for (i = 0; i < MAX_MANT_DIGITS; i++) {
+        decstr[pos++] = mantissa[i] + '0';
         if (i == 0)
-            *cp++ = '.';
+            decstr[pos++] = '.';
     }
-    sprintf(cp, "e%d", exp);
+    snprintf(decstr + pos, 35 - pos, "e%d", exp);
     sscanf(decstr, "%le", &res);
     if (isinf(res))
         return mant_sign ? 2 : 1;
@@ -995,13 +1034,15 @@ int phloat2string(phloat pd, char *buf, int buflen, int base_mode, int digits,
     /* base_mode: 0=only decimal, 1=all bases, 2=SHOW */
     int base = get_base();
     int wsize = effective_wsize();
-    if (base_mode == 1 && base != 10
-            || base_mode == 2 && (base == 2
-                               || base == 8 && disp_r == 2 && (disp_c - 2) * 3 < wsize)) {
+    if (base_mode == 1
+            && (base != 10 || mode_dec_int && mode_appmenu >= MENU_BASE1 && mode_appmenu <= MENU_BASE_DISP)
+        || base_mode == 2
+            && (base == 2 || base == 8 && wsize > 60)) {
+
         uint8 n;
         int inexact, shift;
         bool too_big = false;
-        char binbuf[64];
+        char binbuf[80];
         int binbufptr = 0;
 
         phloat high, low;
@@ -1057,23 +1098,58 @@ int phloat2string(phloat pd, char *buf, int buflen, int base_mode, int digits,
         if (wsize < 64)
             n &= (1ULL << wsize) - 1;
 
-        uint8 mask;
-        mask = disp_r * disp_c >= 64 ? 0 : (uint8) (-1LL << (disp_r * disp_c));
-        if (inexact)
-            mask <<= 1;
-        if (too_big)
-            mask <<= 1;
-        if (base_mode == 2 && base == 2 && (n & mask) != 0) {
-            // Too many bits; won't fit. Use hex instead.
-            string2buf(buf, buflen, &chars_so_far, "hex ", 4);
-            base = 16;
+        if (base_mode == 2 && base == 2) {
+            int maxbits = disp_r * disp_c;
+            if (mode_bin_sep)
+                maxbits = ((maxbits + 1) * 4) / 5;
+            if (inexact)
+                maxbits--;
+            if (too_big)
+                maxbits--;
+            int8 mask = maxbits >= 64 ? 0 : -1LL << maxbits;
+            if ((n & mask) != 0) {
+                // Too many bits; won't fit. Use hex instead.
+                string2buf(buf, buflen, &chars_so_far, "hex ", 4);
+                base = 16;
+            }
         }
-        shift = base == 2 ? 1 : base == 8 ? 3 : 4;
-        while (n != 0) {
-            int digit = (int) (n & (base - 1));
-            char c = digit < 10 ? '0' + digit : 'A' + digit - 10;
-            binbuf[binbufptr++] = c;
-            n >>= shift;
+        if (base == 10) {
+            bool sign = false;
+            if (flags.f.base_signed) {
+                uint8 s = 1ULL << (wsize - 1);
+                if ((n & s) != 0) {
+                    sign = true;
+                    n = ~n + 1;
+                    if (wsize < 64)
+                        n &= (1ULL << wsize) - 1;
+                }
+            }
+            int p = 0;
+            while (n != 0) {
+                if (mode_dec_sep && ++p == 4) {
+                    binbuf[binbufptr++] = sep;
+                    p = 1;
+                }
+                int digit = n % 10;
+                binbuf[binbufptr++] = '0' + digit;
+                n /= 10;
+            }
+            if (sign)
+                binbuf[binbufptr++] = '-';
+        } else {
+            shift = base == 2 ? 1 : base == 8 ? 3 : 4;
+            bool do_sep = base == 2 ? mode_bin_sep : base == 8 ? mode_oct_sep : mode_hex_sep;
+            int p = 0;
+            while (n != 0) {
+                if (do_sep && ++p == 5) {
+                    binbuf[binbufptr++] = ' ';
+                    p = 1;
+                }
+                int digit = (int) (n & (base - 1));
+                char c = digit < 10 ? '0' + digit : 'A' + digit - 10;
+                binbuf[binbufptr++] = c;
+                n >>= shift;
+            }
         }
         if (binbufptr == 0)
             binbuf[binbufptr++] = '0';
@@ -1098,7 +1174,10 @@ int phloat2string(phloat pd, char *buf, int buflen, int base_mode, int digits,
 
 #ifndef BCD_MATH
     double d = to_double(pd);
-    sprintf(decstr, "%.15e", d);
+    snprintf(decstr, 50, "%.*e", MAX_MANT_DIGITS - 2, d);
+    double d2;
+    if (sscanf(decstr, "%le", &d2) != 1 || d != d2)
+        snprintf(decstr, 50, "%.*e", MAX_MANT_DIGITS - 1, d);
 #else
     bid128_to_string(decstr, &pd.val);
 #endif
