@@ -230,7 +230,7 @@ equation_data *new_equation_data(const char *text, int4 length, bool compat_mode
 std::vector<std::string> get_equation_names();
 std::vector<int> get_equation_indexes();
 int store_params();
-bool ensure_var_space(int n);
+bool ensure_var_space(int n, bool do_root = false);
 int store_var(const char *name, int namelength, vartype *value, bool local = false, bool global = false);
 int store_root_var(const char *name, int namelength, vartype *value);
 bool purge_var(const char *name, int namelength, bool force = false, bool global = true, bool local = true);
